/*
 * Digital Compass Gadget
 * Author: Leo Siripornpitak
 * This program uses existing library written by other authors. These
 * libraries will need to be installed before running, otherwise the
 * program will not compile.
 * The section of codes used are available in each library's example.
 * 
 * QMC5883 Module:
 *  @file getCompassdata.ino
 *  @brief Output the compass data
 *  @copyright   Copyright (c) 2010 DFRobot Co.Ltd (http://www.dfrobot.com)
 *  @license     The MIT License (MIT)
 *  @author      [dexian.huang](952838602@qq.com)
 *  @version  V1.0
 *  @date  2017-7-3
 *  @url https://github.com/DFRobot/DFRobot_QMC5883
 * 
 * TM1637 Module:
 *  TM1637.cpp
 *  A library for the 4 digit display
 *
 *  Copyright (c) 2012 seeed technology inc.
 *  Website    : www.seeed.cc
 *  Author     : Frankie.Chu
 *  Create Time: 9 April,2012
 *  url: https://github.com/Seeed-Studio/Grove_4Digital_Display
 */
#include <math.h>
#include <DFRobot_QMC5883.h>
#include "TM1637.h"

#define CLK 2//pins definitions for TM1637 and can be changed to other ports
#define DIO 3

DFRobot_QMC5883 compass(&Wire, /*I2C addr*/HMC5883L_ADDRESS);
TM1637 tm1637(CLK,DIO);

const int Nled =  4;
const int Eled =  5;
const int Sled =  6;
const int Wled =  7;


void setup()
{
  pinMode(Nled, OUTPUT);
  pinMode(Eled, OUTPUT);
  pinMode(Sled, OUTPUT);
  pinMode(Wled, OUTPUT);
  pinMode(8, INPUT);    
  
  Serial.begin(9600);
  while (!compass.begin())
  {
    Serial.println("Could not find a valid 5883 sensor, check wiring!");
    delay(500);
  }

  Serial.println("Initialize HMC5883");

  //Set/get the compass signal gain range, default to be 1.3 Ga
  compass.setRange(HMC5883L_RANGE_1_3GA);
  Serial.print("compass range is:");
  Serial.println(compass.getRange());

  //Set/get measurement mode
  // compass.setMeasurementMode(HMC5883L_CONTINOUS);
  Serial.print("compass measurement mode is:");
  Serial.println(compass.getMeasurementMode());

  //Set/get the data collection frequency of the sensor
  // compass.setDataRate(HMC5883L_DATARATE_15HZ);
  Serial.print("compass data rate is:");
  Serial.println(compass.getDataRate());

  //Get/get sensor status
  compass.setSamples(HMC5883L_SAMPLES_8);
  Serial.print("compass samples is:");
  Serial.println(compass.getSamples());

  tm1637.init();
  tm1637.set(2);//BRIGHT_TYPICAL = 2,BRIGHT_DARKEST = 0,BRIGHTEST = 7;
  
  delay(1000);
}
void loop()
{
  /**
   * @brief  Set declination angle on your location and fix heading
   * @n      You can find your declination on: http://magnetic-declination.com/
   * @n      (+) Positive or (-) for negative
   * @n      For West Lafayette Indiana, declination angle is 4'25W (negative)
   * @n      Formula: (deg + (min / 60.0)) / (180 / PI);
   */
  int bearing_ang = 0;
  int digits[3];
  int temp = 0;

  if(digitalRead(8))
  {
    float declinationAngle = (-4.0 + (25.0 / 60.0)) / (180 / PI);
  compass.setDeclinationAngle(declinationAngle);
  sVector_t mag = compass.readRaw();
  compass.getHeadingDegrees();
  bearing_ang = mag.HeadingDegress;
  Serial.print("X:");
  Serial.print(mag.XAxis);
  Serial.print(" Y:");
  Serial.print(mag.YAxis);
  Serial.print(" Z:");
  Serial.println(mag.ZAxis);
  Serial.print("Degress = ");
  bearing_ang = int(round(bearing_ang));
  Serial.println(bearing_ang);

  digitalWrite(Nled, LOW);
  digitalWrite(Eled, LOW);
  digitalWrite(Sled, LOW);
  digitalWrite(Wled, LOW);
  Serial.print("\ndirection: ");

  if((bearing_ang >= 23) & (bearing_ang <= 67))
  {
    Serial.print("NE" );
    digitalWrite(Nled, HIGH);
    digitalWrite(Eled, HIGH);
  }
  else if((bearing_ang >= 68) & (bearing_ang <= 112))
  {
    Serial.print("E" );
    digitalWrite(Eled, HIGH);
  }
  else if((bearing_ang >= 113) & (bearing_ang <= 157))
  {
    Serial.print("SE" );
    digitalWrite(Sled, HIGH);
    digitalWrite(Eled, HIGH);
  }
  else if((bearing_ang >= 158) & (bearing_ang <= 202))
  {
    Serial.print("S" );
    digitalWrite(Sled, HIGH);
  }
  else if((bearing_ang >= 203) & (bearing_ang <= 247))
  {
    Serial.print("SW" );
    digitalWrite(Sled, HIGH);
    digitalWrite(Wled, HIGH);
  }
  else if((bearing_ang >= 248) & (bearing_ang <= 292))
  {
    Serial.print("W" );
    digitalWrite(Wled, HIGH);
  }
  else if((bearing_ang >= 293) & (bearing_ang <= 337))
  {
    Serial.print("NW" );
    digitalWrite(Nled, HIGH);
    digitalWrite(Wled, HIGH);
  }
  else
  {
    Serial.print("N" );
    digitalWrite(Nled, HIGH);
  }
  Serial.print("\n" );
  
  temp = bearing_ang;
  
  digits[3] = bearing_ang % 10;
  bearing_ang /= 10;
  digits[2] = bearing_ang % 10;
  bearing_ang /= 10;
  digits[1] = bearing_ang % 10;
  bearing_ang /= 10;

  
  if(digits[1] != 0)
  {
    tm1637.display(1,digits[1]);
  }
  else
  {
    tm1637.display(1, 0x7f);
  }
   if((digits[1] == 0) & (digits[2] == 0))
  {
    tm1637.display(2, 0x7f);
  }
  else
  {
    tm1637.display(2,digits[2]);
  }
  tm1637.display(3,digits[3]);

  delay(100);
  }
  else
  {
    digitalWrite(Nled, LOW);
    digitalWrite(Sled, LOW);
    digitalWrite(Eled, LOW);
    digitalWrite(Wled, LOW);
    tm1637.clearDisplay();
  }
}